<?php
/* <meta content="charset=UTF-8"> */

/**
 * ぐるなびWebサービス検索モデル
 *
 */
class Gnaviapi extends Model
{

  /**
   * WebサービスのURI
   * @var string Web API URI
   */
  const APIURI = 'http://api.gnavi.co.jp/ver1/';

  /**
   * WebサービスのアプリケーションID
   * @access private
   * @var string アプリケーションID
   */
  private $_apiKey;

  /**
   * Webサービスへ渡すURIのパラメータ
   * @access private
   * @var array URIパラメータ
   */
  private $_parameters;

  /**
   * カテゴリーリスト
   * @access private
   * @var array カテゴリーリスト
   */
  private $_categoryList = NULL;

  /**
   * アプリケーションIDを設定する
   * @param string $apiKey
   * @return object Gnaviapi
   */
  public function setApiKey($apiKey) {
    $this->_apiKey = $apiKey;
  }

  /**
   * 住所を設定する
   * @param string $address
   * @return object Gnaviapi
   */
  public function setAddress($address) {
    $this->_parameters['address'] = $address;
  }

  /**
   * 大業態コードを設定する
   * @param integer $category
   * @return object Gnaviapi
   */
  public function setCategory($category) {
    $this->_parameters['category_l'] = $category;
  }

  /**
   * 検索開始ページを設定する
   * @param integer $offsetPage
   * @return object Gnaviapi
   */
  public function setOffset($offsetPage) {
    $this->_parameters['offset_page'] = $offsetPage;
  }

  /**
   * URIを組み立てる
   * @param string $searchPath URIPath
   * @return string URI
   */
  private function _build($searchPath = 'RestSearchAPI') {
    $uri = self::APIURI . $searchPath . '/?keyid=' . $this->_apiKey;

    if ($searchPath == 'RestSearchAPI') {
      $parameters = array();
      foreach ($this->_parameters as $key => $val) {
        $parameters[] = $key . '=' . urlencode($val);
      }
      $uri .= '&' . implode('&', $parameters);
    }

    return  $uri;
  }

  /**
   * Web APIから飲食店のデータを取得し返す
   * @return array
   * @throws Exception [取得できない場合]
   */
  public function get() {
    // GETリクエストのタイムアウト設定（5秒）
    $opts = array('http' =>
                  array('timeout' => 5)
            );
    $context  = stream_context_create($opts);
    // Web APIへリクエスト送信
    $xml = file_get_contents($this->_build(), 0, $context);
    // リクエストが失敗したかチェック
    if ($xml === FALSE) {
      throw new Exception('Access Error', 0);
    }

    // Web APIから取得したXMLデータをオブジェクトに代入
    $sxi = simplexml_load_string($xml);

    // Web APIのレスポンスがエラーか確認
    if (isset($sxi->error)) {
      throw new Exception('GnaviApi Error', (int) $sxi->error->code);
    }

    $ret = array();
    $ret['total_hit_count'] = (int) $sxi->total_hit_count;
    $ret['hit_per_page']    = (int) $sxi->hit_per_page;
    $ret['stores']          = array();

    // <BR>と<br>を改行コードに置換
    $search  = array('<BR>', '<br>');
    $replace = array("\n", "\n");
    // 飲食店のデータを処理
    foreach ($sxi->rest as $store) {
      $data = array();
      $data['url']       = str_replace($search, $replace, (string) $store->url);
      $data['name']      = str_replace($search, $replace, (string) $store->name);
      $data['address']   = str_replace($search, $replace, (string) $store->address);
      $data['tel']       = str_replace($search, $replace, (string) $store->tel);
      $data['category']  = str_replace($search, $replace, (string) $store->category);
      $data['opentime']  = str_replace($search, $replace, (string) $store->opentime);
      $data['holiday']   = str_replace($search, $replace, (string) $store->holiday);
      $data['latitude']  = str_replace($search, $replace, (string) $store->latitude);
      $data['longitude'] = str_replace($search, $replace, (string) $store->longitude);

      $ret['stores'][] = $data;
    }

    return $ret;
  }

  /**
   * Web APIからカテゴリーリストを取得し返す
   * @return array
   * @throws Exception [取得できない場合]
   */
  public function getCategoryList() {
    // すでに取得済みの場合はプロパティを返す
    if ($this->_categoryList) {
      return $this->_categoryList;
    }

    // GETリクエストのタイムアウト設定（5秒）
    $opts = array('http' =>
                  array('timeout' => 5)
            );
    $context  = stream_context_create($opts);
    // Web APIへリクエスト送信
    $xml = file_get_contents($this->_build('CategoryLargeSearchAPI'), 0, $context);
    // リクエストが失敗したかチェック
    if ($xml === FALSE) {
      throw new Exception('Access Error', 0);
    }

    // Web APIから取得したXMLデータをオブジェクトに代入
    $sxi = simplexml_load_string($xml);
    if (isset($sxi->error)) {
      throw new Exception('GnaviApi Error', (int) $sxi->error->code);
    }

    $ret = array();
    foreach ($sxi->category_l as $cat) {
      $data = array();
      $data['category_l_code'] = (string) $cat->category_l_code;
      $data['category_l_name'] = (string) $cat->category_l_name;
      $ret[] = $data;
    }

    // 取得したデータをプロパティに保存
    $this->_categoryList = $ret;

    return $ret;
  }
}

/* End of file gnaviapi.php */
/* Location: ./system/application/models/gnaviapi.php */